/*
	NTOLDAPP (Clipboard support for DOS programmes running in NT's VDM.)
	NTOLDAPP.DLL (Win32 DLL part.)
	NTOLDAPP.CPP (Main module.)

	Copyright (C) 1998-01-18 Claus Andr Frber

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#define WINVER 0x0400
#define STRICT

#include <windows.h>
#include <windowsx.h>

#include "ntvdm.h"

bool clip_open;

/*	NTOLDAPP_InitVDD:
	Called by our TSR at initialization.
*/
extern "C" void __export WINAPI NTOLDAPP_InitVDD(void)
{
	return;
}

/*	AL = 00h
	Installation Check
*/
void ntold_installcheck()
{
	setAX(0x0103);	// Windows 3.1 (lie ;-)
	return;
}

/*	AL = 01h
	Open Clipboard
*/
void ntold_open_clipb()
{
	if(!clip_open)
	{
		clip_open = OpenClipboard(NULL);
		setAX(clip_open);
	}
	else
	{
		setAX(0);
	}

	return;
}

/*	AL = 02h
	Empty Clipboard
*/
void ntold_clear_clipb()
{
	setAX( (bool)EmptyClipboard() );
	return;
}

/*	AL = 03h
	Write to clipboard
*/
void ntold_write_clipb()
{
	WORD	format = getDX();
	DWORD	rm_ptr = MAKELONG(getBX(),getES());
	DWORD	size   = MAKELONG(getCX(),getSI())+1;

	HGLOBAL	gmem;	// handle of global mem
	LPVOID	gptr;	// pointer to global mem
	PBYTE	srcptr;	// pointer to source address (converted)

	gmem = GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE, size );
	if(!gmem) { setAX(GetLastError()); return; };

	gptr = GlobalLock( gmem );
	if(!gptr) { setAX(GetLastError()); GlobalFree(gmem); return; };

	srcptr = GetVDMPointer( rm_ptr, size, false );
	memmove( gptr, srcptr, size );
	FreeVDMPointer( rm_ptr, size, srcptr, false  );

	GlobalUnlock(gptr);

	setAX( (bool)SetClipboardData( format, gmem ) );
	return;
}

/*	AL = 04h
	Close Clipboard
*/
void ntold_read_clipb_size()
{
	WORD	format = getDX();

	DWORD	size;

	HANDLE	gmem;

	gmem = GetClipboardData( format );

	size = gmem ? GlobalSize( gmem ) : 0;

	setAX( LOWORD(size) );
	setDX( HIWORD(size) );

	return;
}

/*	AL = 05h
	Read clipboard
*/
void ntold_read_clipb()
{
	WORD	format = getDX();
	DWORD	rm_ptr = MAKELONG(getBX(),getES());

	HANDLE	gmem;
	LPVOID	gptr;
	DWORD	size;

	PBYTE	destptr;

	gmem = GetClipboardData( format );
	if(!gmem) { setAX(false); return; };

	size = GlobalSize( gmem );

	gptr = GlobalLock( gmem );
	if(!gptr) { setAX(false); return; };

	destptr = GetVDMPointer( rm_ptr, size, false );
	memmove( destptr, gptr, size );
	FreeVDMPointer( rm_ptr, size, destptr, false  );

	GlobalUnlock(gptr);

	setAX( true );

	return;
}

/*	AL = 08h
	Close Clipboard
*/
void ntold_close_clipb()
{
	if( clip_open )
	{
		bool succ;
		succ = CloseClipboard();
		clip_open = !succ;
		setAX(succ);
	}
	else
		setAX(0);

	return;
}

/*	AL = 09h
	Don't know exactly what this does.
	Maybe for compatibility with older real mode Windows versions?
	Simply return requested size free memory. (1MB)
*/
void ntold_compact_clipb()
{
	setAX( getCS() );
	setDX( getSI() );
	return;
}

/*	AL = 0Ah
	Don't know exactly what this is supposed to do.
	Just return nothing and pray noone calls it. ;-)
*/
void ntold_get_dev_caps()
{
	return;
}

/*	NTOLDAPP_CallVDD:
	Called for INT 2Fh, AH=17h
*/
extern "C" void __export WINAPI NTOLDAPP_CallVDD(void)
{
	BYTE	fnr = LOBYTE(getDI());

	WORD iax = getAX();
	WORD ibx = getBX();
	WORD icx = getCX();
	WORD idx = getDX();
	WORD isi = getSI();
	WORD idi = getDI();
	WORD ids = getDS();
	WORD ies = getES();

	switch(fnr)
	{
		case 0x00: ntold_installcheck	(); break;
		case 0x01: ntold_open_clipb		(); break;
		case 0x02: ntold_clear_clipb	(); break;
		case 0x03: ntold_write_clipb	(); break;
		case 0x04: ntold_read_clipb_size(); break;
		case 0x05: ntold_read_clipb		(); break;
		case 0x08: ntold_close_clipb	(); break;
		case 0x09: ntold_compact_clipb	(); break;
		case 0x0a: ntold_get_dev_caps	(); break;
	}

	return;
}

/*	ntoldapp_process_attach:
	Called at DLL initialization.
*/
bool ntoldapp_process_attach(HINSTANCE hdll)
{
	clip_open = false;
	return true;
}

/*	ntoldapp_process_attach:
	Called at DLL desinitialization.
	Free NTVDM I/O hooks.
*/
bool ntoldapp_process_detach(HINSTANCE hdll)
{
	if(clip_open)
		CloseClipboard();

	return true;
}

/*	DllEntryPoint:
	Win32 standard DLL entry point.
*/
extern "C"
BOOL WINAPI DllEntryPoint(
	HINSTANCE hdll,		// handle to DLL module
	DWORD fdwReason,	// reason for calling function
	LPVOID			 	// reserved
   )
{
	struct {
		DWORD	event;
		bool(*	entry)(HINSTANCE hdll);
	}
	inittable[] =
	{
		{ DLL_PROCESS_ATTACH, ntoldapp_process_attach },
		{ DLL_PROCESS_DETACH, ntoldapp_process_detach }
	};

	for (int x = 0; x < sizeof inittable /sizeof inittable[0]; x++ )
		if( inittable[x].event == fdwReason )
			return inittable[x].entry(hdll);
	return true;
}

